library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

-- entity
-------------------------------------------------------------------------------

entity videoreader is  --USER--

  port
  (
    IPD         : in    std_logic_vector(7 downto 0);
    ICLK        : in    std_logic;
    IDQ         : in    std_logic;
    IGPH        : in    std_logic;
    IGPV        : in    std_logic;

    pix_count   : in  std_logic_vector(10 downto 0);
    active       : in  std_logic;
    frame_counter : in std_logic_vector(1 downto 0);
    
    position      : out   std_logic_vector(15 downto 0);
    length        : out   std_logic_vector(15 downto 0);
    
    strength : out std_logic_vector(17 downto 0);
    threshold : in std_logic_vector(7 downto 0)
    
    );
 
end entity videoreader; --USER-- 

-------------------------------------------------------------------------------
-- architecture
-------------------------------------------------------------------------------

architecture imp of videoreader is --USER-- 

signal active_block_flag        : std_logic;  -- place pixels in registers
signal counter_valid            : std_logic_vector(1 downto 0);
signal temp_position            : std_logic_vector(15 downto 0) := "0000000000000000";
signal last_position            : std_logic_vector(15 downto 0) := "0000000000000000";
signal hold_position            : std_logic :='0';
signal temp_length              : std_logic_vector(15 downto 0) := "0000000000000000";
signal hold_length              : std_logic_vector(15 downto 0) := "0000000000000000";
signal last_length              : std_logic_vector(15 downto 0) := "0000000000000000";
signal reg_counter              : std_logic_vector(2 downto 0);
signal check_0                  : std_logic := '0';
signal check_1                  : std_logic := '0';
signal vid_reg0                 : std_logic_vector(7 downto 0) := "00000000";
signal vid_reg1                 : std_logic_vector(7 downto 0) := "00000000";
signal vid_reg2                 : std_logic_vector(7 downto 0) := "00000000";
signal vid_reg3                 : std_logic_vector(7 downto 0) := "00000000";
signal vid_reg4                 : std_logic_vector(7 downto 0) := "00000000";
signal vid_reg5                 : std_logic_vector(7 downto 0) := "00000000";
signal vid_reg6                 : std_logic_vector(7 downto 0) := "00000000";
signal vid_reg7                 : std_logic_vector(7 downto 0) := "00000000";
signal compare_reg              : std_logic;
signal compare_length           : std_logic;
signal current_state            : std_logic_vector(3 downto 0) := "1110";
signal next_state               : std_logic_vector(3 downto 0);
signal state_reset              : std_logic := '0';
signal dont_write               : std_logic := '0';
signal filter_safe              : std_logic := '0';
signal fstream                  : std_logic := '0';
signal region1_pix              : std_logic_vector(7 downto 0) := "00000000";
signal region2_pix              : std_logic_vector(7 downto 0) := "00000000";
signal region3_pix              : std_logic_vector(7 downto 0) := "00000000";

signal t_strength : std_logic_vector(17 downto 0);



--constant threshold              : std_logic_vector(7 downto 0) := "01010000";
--constant stateA                 : std_logic_vector(3 downto 0) := "0000";
constant stateB                 : std_logic_vector(3 downto 0) := "0001";
constant stateC                 : std_logic_vector(3 downto 0) := "0010";
constant stateD                 : std_logic_vector(3 downto 0) := "0011";
constant stateE                 : std_logic_vector(3 downto 0) := "0100";
constant stateF                 : std_logic_vector(3 downto 0) := "0101";
constant stateG                 : std_logic_vector(3 downto 0) := "0110";
constant stateH                 : std_logic_vector(3 downto 0) := "0111";
constant stateI                 : std_logic_vector(3 downto 0) := "1000";
constant stateValid             : std_logic_vector(3 downto 0) := "1101";
constant stateInvalid           : std_logic_vector(3 downto 0) := "1110";


begin


-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- This is an attempt at a filter. This filter will examine the second bit in a
-- series of three and allow it to 'pass' if the two bits around it are 'on.'
-------------------------------------------------------------------------------
--first set up a minibuffer
  process (ICLK, IDQ)
  begin
    if ICLK'event and ICLK = '1' and IDQ = '1' then
      if active = '1' then
        vid_reg0 <= IPD;
        vid_reg1 <= vid_reg0;
        vid_reg2 <= vid_reg3;
      elsif active = '0' then
        vid_reg0 <= "00000000";
        vid_reg1 <= "00000000";
        vid_reg2 <= "00000000";
      end if;
    end if;
  end process;

 -- Continually running statement that turns on the middle pixel if surrounded
 -- by two on pixels
  fstream <= '1' and IDQ and active when (vid_reg1 < threshold or (vid_reg0 < threshold and vid_reg2 < threshold))
             else '0';
-------------------------------------------------------------------------------
  compare_reg <= fstream;
--  compare_reg <= '1' and IDQ and active when IPD < threshold else '0';
  compare_length <= '1' when temp_length > last_length else '0';
-------------------------------------------------------------------------------

-- active_block_flag indicates whether we are in the middle of a region that is
-- entirely below the threshold
  process (ICLK)
  begin
    if ICLK'event and ICLK='1' and IDQ = '1' then
      if active_block_flag = '0' then 
        if compare_reg = '1' then
          active_block_flag <= '1';
        end if;
      elsif active_block_flag = '1' then
        if compare_reg = '0' then
          active_block_flag <= '0';
        end if;
      end if;
      if active = '0' then
        active_block_flag <= '0';
      end if;
    end if;
  end process;

--These next few processes control the length output signal
  process(ICLK)
  begin
     if ICLK'event and ICLK='1' and IDQ = '1' then
       if active = '0' then
         t_strength <= X"0000" &"00";
       else
         t_strength <= t_strength + IPD;  
       end if;
     end if;
  end process;

  process(ICLK)
    begin
      if ICLK'event and ICLK='1' and IDQ = '1' then
        if active = '1' then
          strength <= t_strength;
        end if;
      end if;
    end process;

  
-- increment last length appropriately
  
  process (ICLK)
  begin
    if ICLK'event and ICLK = '1' and IDQ='1' then
      if compare_reg = '1' then 
        last_length <= last_length+1;
      elsif active_block_flag = '0' then
        last_length <= X"0000";
      end if;
      if active = '0' then
        last_length <= X"0000";
      end if;
    end if;
  end process;

-- assign last length to temp length if last length is longer
  process (ICLK)
  begin
    if ICLK'event and ICLK = '1' and IDQ = '1' then
      if compare_length = '0' then
        temp_length <= last_length;
      end if;
      if active = '0' then
        temp_length <= X"0000";
      end if;
    end if;
  end process;

-- output the length signal to opb_bus
  process (ICLK)
  begin
    if ICLK'event and ICLK = '1' and IDQ = '1' then
      if active = '1' then
        length <= '0' & temp_length(15 downto 1);        
--        length <= temp_length;
      end if;
    end if;
  end process;

-- flags controlling the position signal

-- keep updating to last position before active block and then holds that value
-- once we start looking at active block



  process (ICLK)
  begin
    if ICLK'event and ICLK = '1' and IDQ = '1' then
      if active_block_flag = '0' and compare_reg = '0' then
        last_position <= "00000" & pix_count;
      end if;
    end if;
  end process;

-- if the block we are looking at now is longer than any previous block then
-- assign the starting position of the current block (last position) to temp
  process (ICLK)
  begin
    if ICLK'event and ICLK = '1' and IDQ = '1' then
      if compare_length = '0' then
        temp_position <= last_position;
      end if;
      if active = '0' then
        temp_position <= X"0000";
      end if;
    end if;
  end process;
  
-- output position to opb_bus
   process (ICLK)
   begin
     if ICLK'event and ICLK = '1' then
       if active='1' then
         position <= '0' & temp_position(15 downto 1);
--         position <= temp_position;         
       end if;
     end if;
   end process;

  
end architecture imp;
  
